
###Read data files

###To load data file
data_all <- read.table("        ", header = T, sep = ",", na.strings = "NA")

###To load spatial neighborhood data
neighbor_data <- read.table("       ", header = F, sep = ",", na.strings = "NA")

###To load distance between two spatial units
distance <- read.table("     ", header = T, sep = ",", na.strings = "NA")


###To determine an appropriate temporal window

data_all$rate <- data_all$age10_14

number_area <- max(data_all$FID) + 1

number_duration <- max(data_all$year)- min(data_all$year) + 1

total_lag_number <-  number_duration - 1

sig_level <- 0.05

cor_lag <- pvalue <- numeric(total_lag_number)


require(TSA)

for (i in 1:total_lag_number) {

     rate_original <- data_all$rate

     rate_lag <- zlag(rate_original, number_area*i)

     result <- cor.test(rate_original, rate_lag, use="pairwise.complete.obs")

     pvalue[i] <- result$p.value

     cor_lag[i] <- result$estimate

    }

lag_decision_table <- cbind(cor_lag, pvalue)

lag_decision_table <- as.data.frame(lag_decision_table)

names(lag_decision_table) <- c("Correlation","P-value")

row.names(lag_decision_table) <- c("lag 1","lag 2","lag 3","lag 4","lag 5","lag 6","lag 7","lag 8")

write.csv(lag_decision_table, file = "M:/104pe/RingMap/pgm/TWdata/results/lag_|age10_14.csv")

lag_number <- min(which(pvalue > sig_level)) - 1




###To calculate spatial-temporal Gi-star statistics


for (kkk in 4:8) {

lag_number <- kkk

data_all$rate <- data_all$age10_14

number_area <- max(data_all$FID) + 1

number_duration <- max(data_all$year)- min(data_all$year) + 1

indicator_start <- min(data_all$FID + 1)

indicator_end <- max(data_all$FID + 1)

number_duration <- max(data_all$year)- min(data_all$year) + 1

data_all_mat <- matrix(NA,nrow = number_area, ncol = number_duration)

id_area <- data_all$FID + 1

indicator_start_all <- which(id_area == 1)

indicator_end_all <- which(id_area == number_area)

for (j in 1:(length(indicator_start_all))) {
    for (i in indicator_start_all[j]:indicator_end_all[j]) {

        data_all_mat[id_area[i], j] <- data_all$rate[i]

   }
}


neighbor_mat <- matrix(0,nrow = number_area, ncol = number_area)

for (i in 1:number_area) {
   for (j in 1:length(neighbor_data)) {

    if (!is.na(neighbor_data[i, j]))
        { neighbor_mat[neighbor_data$V1[i], neighbor_data[i, j]] = 1}
#    else
#        { neighbor_mat[i, j] = 0}
  }
}


distance_mat <- diag(10^-6, number_area, number_area)

for (i in 1:length(distance$INPUT_FID)) {
    k <- distance$INPUT_FID[i] + 1
    m <- distance$NEAR_FID[i] + 1
    l <- distance$DISTANCE[i]

    distance_mat[k, m] <- l

}

nei_dis_mat <-  neighbor_mat *  distance_mat
      
weight_mat <- 1-(nei_dis_mat/rowSums(nei_dis_mat))


for (i in 1:number_area) {
   for (j in 1:number_area) {

    if (nei_dis_mat[i, j] == 0)
        { weight_mat[i, j] <- 0}
  }
   
    if (length(which(weight_mat[i,] == 0))== number_area)
        { weight_mat[i, i] <- 1}

 }


weight_lag_array <- array(NA, c(nrow(weight_mat),nrow(weight_mat)*(lag_number+1) ))

for (i in 1:(lag_number+1)) {

     nam <- paste("weight_lag",i, sep="")

     assign(nam, (weight_mat)^(i))

     weight_lag_array[,((nrow(weight_mat)*(i-1))+1):(nrow(weight_mat)*(i))] <- (weight_mat)^(i)

}


mean_mat <- matrix(NA,nrow = number_area, ncol = ncol(data_all_mat))

std_mat  <- matrix(NA,nrow = number_area, ncol = ncol(data_all_mat))

for (i in 1:ncol(data_all_mat)) {

    mean_mat[indicator_start:indicator_end, i] <-  rep(mean(data_all_mat[indicator_start:indicator_end, max(1, (i-lag_number)):i], na.rm = T),(indicator_end - indicator_start + 1)) 

    std_mat[indicator_start:indicator_end, i] <-  rep(sd(data_all_mat[indicator_start:indicator_end, max(1, (i-lag_number)):i], na.rm = T),(indicator_end - indicator_start + 1)) 


}


mean_mat <- replace(mean_mat, is.nan(mean_mat), NA)

mGistar_mat <- matrix(NA,nrow = number_area, ncol = number_duration)
numerator1 <-  matrix(NA,nrow = 1, ncol = lag_number+1)
numerator2 <-  matrix(NA,nrow = 1, ncol = lag_number+1)
denominator1 <-  matrix(NA,nrow = 1, ncol = lag_number+1)
denominator2 <-  matrix(NA,nrow = 1, ncol = lag_number+1)



for (i in 1:number_area) {

    for (j in 1:ncol(data_all_mat)) {

     if (j <= lag_number+1) {

    numerator1 <-  matrix(NA,nrow = 1, ncol = j)
    numerator2 <-  matrix(NA,nrow = 1, ncol = j)
    denominator1 <-  matrix(NA,nrow = 1, ncol = j)
    denominator2 <-  matrix(NA,nrow = 1, ncol = j)
     }

     if (j > lag_number+1) {

    numerator1 <-  matrix(NA,nrow = 1, ncol = lag_number+1)
    numerator2 <-  matrix(NA,nrow = 1, ncol = lag_number+1)
    denominator1 <-  matrix(NA,nrow = 1, ncol = lag_number+1)
    denominator2 <-  matrix(NA,nrow = 1, ncol = lag_number+1)
     }


# numerator of Gistar

    if ( i >= indicator_start & i <= indicator_end & !is.na(data_all_mat[i,j])) {

      for (k in 1:(min(j,lag_number+1))) {

          numerator1[1,k] <-  weight_lag_array[i,(indicator_start + (number_area*(k-1))):(indicator_end + (number_area*(k-1)))] %*% data_all_mat[indicator_start:indicator_end ,-(k-1-j)]
          
          numerator2[1,k] <-  sum(weight_lag_array[i,(indicator_start+(number_area*(k-1))):(indicator_end+(number_area*(k-1)))], na.rm=T)

          denominator1[1,k] <- sum((weight_lag_array[i,(indicator_start+(number_area*(k-1))):(indicator_end+(number_area*(k-1)))])^2, na.rm=T)

          denominator2[1,k] <- sum(weight_lag_array[i,(indicator_start+(number_area*(k-1))):(indicator_end+(number_area*(k-1)))], na.rm=T)

     }

#      numerator <- sum(numerator1, na.rm=T)- (mean_mat_all[i,j]*sum(numerator2, na.rm=T))

      numerator <- sum(numerator1, na.rm=T)- (mean_mat[i,j]*sum(numerator2, na.rm=T))

      if ( j <= lag_number+1 ) {
      denominator <-  std_mat[i,j] * sqrt(((((indicator_end-indicator_start+1)*(j)) * (sum(denominator1, na.rm=T)))- (sum(denominator2, na.rm=T)^2))/ (((indicator_end-indicator_start+1)*(j))-1))
       }
      else
      {denominator <-  std_mat[i,j] * sqrt(((((indicator_end-indicator_start+1)*(lag_number+1)) * (sum(denominator1, na.rm=T)))- (sum(denominator2, na.rm=T)^2))/ (((indicator_end-indicator_start+1)*(lag_number+1))-1))
      }

      mGistar_mat[i,j] <- numerator/denominator

    }

  }
}



mGistar_mat_dat <- as.data.frame(mGistar_mat)

names(mGistar_mat_dat) <- c("2009","2010","2011","2012","2013","2014","2015","2016","2017")

row.names(mGistar_mat_dat) <- c("Keelung City","Taipei City","New Taipei City","Taoyuan City","Hsinchu City","Hsinchu County","Miaoli County","Taichung City","Nantou County","Changhua County","Yunlin County","Chiayi City","Chiayi County","Tainan City","Kaohsiung City","Pingtung County","Yilan County","Hualien County","Taitung County","Penghu County","Kinmen County","Lienchiang County")


### To write output file

filename = paste0("        ",kkk,".csv")

write.csv(mGistar_mat_dat, file = filename)

}







========================================================================================================================================








 














